#include "stc.h"
#include "usb.h"
#include "iap.h"
#include "util.h"
#include "dfu.h"

DWORD xdata DfuFlag _at_ 0x1ffc;

static void delay()
{
    int i;
    
    for (i = 0; i < 3000; i++);
}

void dfu_check()
{
    P3M1 &= ~0x08;
    P3PU |= 0x08;
    _nop_();
    _nop_();
    _nop_();
    _nop_();
    delay();
    
    if ((DFU_FORCEPIN != 0) &&
        (DfuFlag != DFU_TAG) &&
        (*(BYTE code *)(LDR_SIZE) == 0x02) &&
        (*(WORD code *)(LDR_SIZE + 1) >= LDR_SIZE + 3))
    {
        P3M1 |= 0x08;
        P3PU = 0x00;
        ((void (code *)())(LDR_SIZE))();
    }
    
    P3M1 |= 0x08;
    P3PU = 0x00;
    DfuFlag = 0;
}

void dfu_events()
{
    BYTE cmd;
    WORD addr;
    BYTE size;
    BYTE *ptr;
    BYTE status;

    if (!bUsbOutReady)
        return;

    status = STATUS_ERRORWRAP;
    if ((UsbOutBuffer[0] == '#') &&
        (UsbOutBuffer[62] == '$') &&
        (chksum(UsbOutBuffer, 64) == 0))
    {
        cmd = UsbOutBuffer[1];
        addr = *(WORD *)&UsbOutBuffer[4];
        size = UsbOutBuffer[6];
        ptr = &UsbOutBuffer[7];
        status = STATUS_OK;
        
        memset(UsbInBuffer, 0, 64);

        switch (cmd)
        {
        case DFU_CMD_CONNECT:
            UsbInBuffer[2] = 2;
            UsbInBuffer[3] = LDR_VERSION >> 8;
            UsbInBuffer[4] = LDR_VERSION;
            break;
        case DFU_CMD_READ:
#ifdef DEBUG
            if (size > 59)
            {
                status = STATUS_OUTOFRANGE;
            }
            else
            {
                UsbInBuffer[2] = size;
                ptr = &UsbInBuffer[3];
                while (size--)
                {
                    *ptr++ = iap_read_byte(addr++);
                }
            }
#else
            status = STATUS_ERRORCMD;
#endif
            break;
        case DFU_CMD_PROGRAM:
            if (size > 55)
            {
                status = STATUS_OUTOFRANGE;
            }
            else
            {
                while (size--)
                {
                    if (!iap_write_byte(addr, *ptr))
                    {
                        status = STATUS_PROGRAMERR;
                        break;
                    }
                    addr++;
                    ptr++;
                }
            }
            break;
        case DFU_CMD_ERASE:
            addr = 0;
            while (addr < 0xf000)
            {
                iap_erase_page(addr);
                addr += 0x200;
            }
            break;
        case DFU_CMD_REBOOT:
            usb_OUT_done();

            USBCON = 0x00;
            USBCLK = 0x00;
            IRC48MCR = 0x00;
            sleep_ms(10);
            IAP_CONTR = 0x20;
            while (1);
            break;
        default:
            status = STATUS_ERRORCMD;
            break;
        }
    }

    if (status == STATUS_ERRORWRAP)
    {
        if (USER_STCISPCMD != NULL)
        {
            if (strcmp(UsbOutBuffer, USER_STCISPCMD) == 0)
            {
                usb_write_reg(OUTCSR1, 0);
                
                USBCON = 0x00;
                USBCLK = 0x00;
                IRC48MCR = 0x00;
                
                sleep_ms(10);
                
                IAP_CONTR = 0x60;
                while (1);
            }
        }
    }
    else
    {
        UsbInBuffer[0] = '@';
        UsbInBuffer[1] = status;
        UsbInBuffer[62] = '$';
        UsbInBuffer[63] = -chksum(UsbInBuffer, 63);
        
        usb_IN();
    }
    
    usb_OUT_done();
}
